#include "servicesvoiceservice.h"
#include "servicesvoiceservicecallcontrol.h"
#include "servicesvoiceservicecalllog.h"
#include "servicesvoiceservicedect.h"

/**********************************************************************************************************************************
*                                            OBJ & PARAM DEFINITION
***********************************************************************************************************************************/
static int get_ServicesVoiceHalInit(char *refparam, struct dmctx *ctx, void *data, char *instance, char **value)
{
    	struct uci_section *s;

	if ((s = is_dmmap_section_exist("dmmap_voice_hal_init", "general")))
	    *value = dmuci_get_value_by_section_fallback_def(s, "init_done", "0");

	return 0;
}

static int set_ServicesVoiceHalInit(char *refparam, struct dmctx *ctx, void *data, char *instance, char *value, int action)
{
	bool b;
	struct uci_section *dmmap = NULL, *s = NULL, *stmp = NULL;

	switch (action) {
		case VALUECHECK:
			if (bbfdm_validate_boolean(ctx, value))
				return FAULT_9007;
		case VALUESET:
			if (!(dmmap = is_dmmap_section_exist("dmmap_voice_hal_init", "general"))) {
				dmuci_add_section("voice_hal_init", "general", &s);
				dmuci_add_section_bbfdm("dmmap_voice_hal_init", "general", &dmmap);
				dmuci_set_value_by_section(dmmap, "section_name", section_name(s));
			}
			string_to_bool(value, &b);
			uci_foreach_sections_safe("voice_hal_init", "general", stmp, s) {
				get_dmmap_section_of_config_section("dmmap_voice_hal_init", "general", section_name(s), &dmmap);
				dmuci_set_value_by_section(dmmap, "init_done", b ? "1" : "0");
				dmuci_set_value_by_section(s, "init_done", b ? "1" : "0");
			}

			return 0;
	}
	return 0;
}

DM_MAP_OBJ tDynamicObj[] = {
/* parentobj, nextobject, parameter */
{"Device.Services.", NULL, tServicesVoiceHalParams},
{"Device.Services.VoiceService.{i}.CallLog.{i}.", NULL, tIOPSYS_VoiceServiceCallLogParams},
{"Device.Services.VoiceService.{i}.CallLog.{i}.Session.{i}.Source.RTP.", NULL, tIOPSYS_VoiceServiceCallLogSessionSourceRTPParams},
{"Device.Services.VoiceService.{i}.CallLog.{i}.Session.{i}.Destination.RTP.", NULL, tIOPSYS_VoiceServiceCallLogSessionDestinationRTPParams},
{"Device.Services.VoiceService.{i}.DECT.Portable.{i}.", NULL, tIOPSYS_VoiceServiceDECTPortableParams},
{"Device.Services.VoiceService.{i}.CallControl.Extension.{i}.", NULL, tIOPSYS_VoiceServiceCallControlExtensionParams},
{0}
};

DMLEAF tServicesVoiceHalParams[] = {
{BBF_VENDOR_PREFIX"VoiceHalInit", &DMWRITE, DMT_BOOL, get_ServicesVoiceHalInit, set_ServicesVoiceHalInit, BBFDM_BOTH},
{0}
};
